# Copyright 2016 Futurewei. All rights reserved.
# Copyright 2017 Intel Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions as lib_exc
from tempest.lib.services.network import base


class PortChainClient(base.BaseNetworkClient):

    def create_port_chain(self, **kwargs):
        uri = '/sfc/port_chains'
        post_data = {'port_chain': kwargs}
        return self.create_resource(uri, post_data)

    def update_port_chain(self, pc_id, **kwargs):
        uri = '/sfc/port_chains/%s' % pc_id
        post_data = {'port_chain': kwargs}
        return self.update_resource(uri, post_data)

    def show_port_chain(self, pc_id, **fields):
        uri = '/sfc/port_chains/%s' % pc_id
        return self.show_resource(uri, **fields)

    def delete_port_chain(self, pc_id):
        uri = '/sfc/port_chains/%s' % pc_id
        return self.delete_resource(uri)

    def list_port_chains(self, **filters):
        uri = '/sfc/port_chains'
        return self.list_resources(uri, **filters)

    def is_resource_deleted(self, id):
        try:
            self.show_port_chain(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'sfc'


class PortPairGroupClient(base.BaseNetworkClient):

    def create_port_pair_group(self, **kwargs):
        uri = '/sfc/port_pair_groups'
        post_data = {'port_pair_group': kwargs}
        return self.create_resource(uri, post_data)

    def update_port_pair_group(self, pg_id, **kwargs):
        uri = '/sfc/port_pair_groups/%s' % pg_id
        post_data = {'port_pair_group': kwargs}
        return self.update_resource(uri, post_data)

    def show_port_pair_group(self, pg_id, **fields):
        uri = '/sfc/port_pair_groups/%s' % pg_id
        return self.show_resource(uri, **fields)

    def delete_port_pair_group(self, pg_id):
        uri = '/sfc/port_pair_groups/%s' % pg_id
        return self.delete_resource(uri)

    def list_port_pair_groups(self, **filters):
        uri = '/sfc/port_pair_groups'
        return self.list_resources(uri, **filters)

    def is_resource_deleted(self, id):
        try:
            self.show_port_pair_group(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'sfc'


class PortPairClient(base.BaseNetworkClient):

    def create_port_pair(self, **kwargs):
        uri = '/sfc/port_pairs'
        post_data = {'port_pair': kwargs}
        return self.create_resource(uri, post_data)

    def update_port_pair(self, pp_id, **kwargs):
        uri = '/sfc/port_pairs/%s' % pp_id
        post_data = {'port_pair': kwargs}
        return self.update_resource(uri, post_data)

    def show_port_pair(self, pp_id, **fields):
        uri = '/sfc/port_pairs/%s' % pp_id
        return self.show_resource(uri, **fields)

    def delete_port_pair(self, pp_id):
        uri = '/sfc/port_pairs/%s' % pp_id
        return self.delete_resource(uri)

    def list_port_pairs(self, **filters):
        uri = '/sfc/port_pairs'
        return self.list_resources(uri, **filters)

    def is_resource_deleted(self, id):
        try:
            self.show_port_pair(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'sfc'


class ServiceGraphClient(base.BaseNetworkClient):

    def create_service_graph(self, **kwargs):
        uri = '/sfc/service_graphs'
        post_data = {'service_graph': kwargs}
        return self.create_resource(uri, post_data)

    def update_service_graph(self, pp_id, **kwargs):
        uri = '/sfc/service_graphs/%s' % pp_id
        post_data = {'service_graph': kwargs}
        return self.update_resource(uri, post_data)

    def show_service_graph(self, pp_id, **fields):
        uri = '/sfc/service_graphs/%s' % pp_id
        return self.show_resource(uri, **fields)

    def delete_service_graph(self, pp_id):
        uri = '/sfc/service_graphs/%s' % pp_id
        return self.delete_resource(uri)

    def list_service_graphs(self, **filters):
        uri = '/sfc/service_graphs'
        return self.list_resources(uri, **filters)

    def is_resource_deleted(self, id):
        try:
            self.show_service_graph(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'sfc'



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest.lib import exceptions

TempestException = exceptions.TempestException


class QoSLimitReached(TempestException):
    message = "Limit reached, limit = %(limit)d"


class SocketConnectionRefused(TempestException):
    message = "Unable to connect to %(host)s port %(port)d:Connection Refused"


class ConnectionTimeoutException(TempestException):
    message = "Timeout connecting to %(host)s port %(port)d"


class FileCreationFailedException(TempestException):
    message = "File %(file)s has not been created or has the wrong size"



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants as lib_constants
from oslo_log import log
from paramiko import ssh_exception as ssh_exc
from tempest.common import utils as tempest_utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.common import ip
from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin.common import utils
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF

LOG = log.getLogger(__name__)


def turn_nic6_on(ssh, ipv6_port):
    """Turns the IPv6 vNIC on

    Required because guest images usually set only the first vNIC on boot.
    Searches for the IPv6 vNIC's MAC and brings it up.

    @param ssh: RemoteClient ssh instance to server
    @param ipv6_port: port from IPv6 network attached to the server
    """
    ip_command = ip.IPCommand(ssh)
    nic = ip_command.get_nic_name_by_mac(ipv6_port['mac_address'])

    # NOTE(slaweq): on RHEL based OS ifcfg file for new interface is
    # needed to make IPv6 working on it, so if
    # /etc/sysconfig/network-scripts directory exists ifcfg-%(nic)s file
    # should be added in it
    if sysconfig_network_scripts_dir_exists(ssh):
        try:
            ssh.execute_script(
                'echo -e "DEVICE=%(nic)s\\nNAME=%(nic)s\\nIPV6INIT=yes" | '
                'tee /etc/sysconfig/network-scripts/ifcfg-%(nic)s; '
                'nmcli connection reload' % {'nic': nic},
                become_root=True)
            ssh.execute_script('nmcli connection up %s' % nic,
                               become_root=True)
        except lib_exc.SSHExecCommandFailed as e:
            # NOTE(slaweq): Sometimes it can happen that this SSH command
            # will fail because of some error from network manager in
            # guest os.
            # But even then doing ip link set up below is fine and
            # IP address should be configured properly.
            LOG.debug("Error during restarting %(nic)s interface on "
                      "instance. Error message: %(error)s",
                      {'nic': nic, 'error': e})
    ip_command.set_link(nic, "up")


def sysconfig_network_scripts_dir_exists(ssh):
    return "False" not in ssh.execute_script(
        'test -d /etc/sysconfig/network-scripts/ || echo "False"')


class IPv6Test(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    ipv6_ra_mode = 'slaac'
    ipv6_address_mode = 'slaac'

    @classmethod
    @tempest_utils.requires_ext(extension="router", service="network")
    def resource_setup(cls):
        super(IPv6Test, cls).resource_setup()
        cls.reserve_external_subnet_cidrs()
        cls._setup_basic_resources()

    @classmethod
    def _setup_basic_resources(cls):
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair()
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name('secgroup'))
        cls.create_loginable_secgroup_rule(secgroup_id=cls.secgroup['id'])
        cls.create_pingable_secgroup_rule(secgroup_id=cls.secgroup['id'])

    def _test_ipv6_address_configured(self, ssh_client, vm, ipv6_port):
        ipv6_address = ipv6_port['fixed_ips'][0]['ip_address']
        ip_command = ip.IPCommand(ssh_client)

        def guest_has_address(expected_address):
            ip_addresses = [a.address for a in ip_command.list_addresses()]
            for ip_address in ip_addresses:
                if expected_address in ip_address:
                    return True
            return False

        try:
            # Set NIC with IPv6 to be UP and wait until IPv6 address will be
            # configured on this NIC
            turn_nic6_on(ssh_client, ipv6_port)
            # And check if IPv6 address will be properly configured on this NIC
            utils.wait_until_true(
                lambda: guest_has_address(ipv6_address),
                timeout=120,
                exception=RuntimeError(
                    "Timed out waiting for IP address {!r} to be configured "
                    "in the VM {!r}.".format(ipv6_address, vm['id'])))
        except (lib_exc.SSHTimeout, ssh_exc.AuthenticationException) as ssh_e:
            LOG.debug(ssh_e)
            self._log_console_output([vm])
            self._log_local_network_status()
            raise

    def _test_ipv6_hotplug(self, ra_mode, address_mode):
        ipv6_networks = [self.create_network() for _ in range(2)]
        for net in ipv6_networks:
            subnet = self.create_subnet(
                network=net, ip_version=6,
                ipv6_ra_mode=ra_mode, ipv6_address_mode=address_mode)
            self.create_router_interface(self.router['id'], subnet['id'])

        server_kwargs = {
            'flavor_ref': CONF.compute.flavor_ref,
            'image_ref': CONF.compute.image_ref,
            'key_name': self.keypair['name'],
            'networks': [
                {'uuid': self.network['id']},
                {'uuid': ipv6_networks[0]['id']}],
            'security_groups': [{'name': self.secgroup['name']}],
        }
        vm = self.create_server(**server_kwargs)['server']
        self.wait_for_server_active(vm)
        self.wait_for_guest_os_ready(vm)
        ipv4_port = self.client.list_ports(
            network_id=self.network['id'],
            device_id=vm['id'])['ports'][0]
        fip = self.create_floatingip(port=ipv4_port)
        ssh_client = ssh.Client(
            fip['floating_ip_address'], CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        ipv6_port = self.client.list_ports(
            network_id=ipv6_networks[0]['id'],
            device_id=vm['id'])['ports'][0]
        self._test_ipv6_address_configured(ssh_client, vm, ipv6_port)

        # Now remove this port IPv6 port from the VM and attach new one
        self.delete_interface(vm['id'], ipv6_port['id'])

        # And plug VM to the second IPv6 network
        ipv6_port = self.create_port(ipv6_networks[1])
        self.create_interface(vm['id'], ipv6_port['id'])
        ip.wait_for_interface_status(
            self.os_primary.interfaces_client, vm['id'],
            ipv6_port['id'], lib_constants.PORT_STATUS_ACTIVE,
            ssh_client=ssh_client, mac_address=ipv6_port['mac_address'])
        self._test_ipv6_address_configured(ssh_client, vm, ipv6_port)

    @decorators.idempotent_id('b13e5408-5250-4a42-8e46-6996ce613e91')
    def test_ipv6_hotplug_slaac(self):
        self._test_ipv6_hotplug("slaac", "slaac")

    @decorators.idempotent_id('9aaedbc4-986d-42d5-9177-3e721728e7e0')
    def test_ipv6_hotplug_dhcpv6stateless(self):
        self._test_ipv6_hotplug("dhcpv6-stateless", "dhcpv6-stateless")



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest import config

from neutron_tempest_plugin.fwaas.common import fwaas_v2_client

CONF = config.CONF


class BaseFWaaSTest(fwaas_v2_client.FWaaSClientMixin, base.BaseNetworkTest):

    @classmethod
    def skip_checks(cls):
        super(BaseFWaaSTest, cls).skip_checks()
        msg = None
        if not CONF.fwaas.run_fwaas_tests:
            msg = ("Running of fwaas related tests is disabled in "
                   "plugin configuration.")
        if msg:
            raise cls.skipException(msg)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import qos as qos_apidef
from neutron_lib.db import constants as db_const
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base

LONG_NAME_NG = 'z' * (db_const.NAME_FIELD_SIZE + 1)
LONG_DESCRIPTION_NG = 'z' * (db_const.LONG_DESCRIPTION_FIELD_SIZE + 1)
LONG_TENANT_ID_NG = 'z' * (db_const.PROJECT_ID_FIELD_SIZE + 1)


class QosNegativeTestJSON(base.BaseAdminNetworkTest):

    required_extensions = [qos_apidef.ALIAS]

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce555-d3b3-11e5-950a-54ee757c77da')
    def test_add_policy_with_too_long_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          LONG_NAME_NG, 'test policy desc1', False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce444-d3b3-11e5-950a-54ee747c99db')
    def test_add_policy_with_too_long_description(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          'test-policy', LONG_DESCRIPTION_NG, False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce444-d3b3-11e5-950a-54ee757c77dc')
    def test_add_policy_with_too_long_tenant_id(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          'test-policy', 'test policy desc1',
                          False, LONG_TENANT_ID_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0e85f3e4-7a93-4187-b847-8f4e835aae1b')
    def test_update_policy_with_too_long_name(self):
        policy = self.create_qos_policy(name='test', description='test policy',
                                        shared=False)
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_qos_policy, policy['id'],
                          name=LONG_NAME_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('925c7eaf-474b-4a02-a4ba-76a9f82bc45a')
    def test_update_policy_with_too_long_description(self):
        policy = self.create_qos_policy(name='test', description='test policy',
                                        shared=False)
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_qos_policy, policy['id'],
                          description=LONG_DESCRIPTION_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('88b54ab0-804b-446c-bc19-8e54222d70ef')
    def test_get_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.show_qos_policy, non_exist_id)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('21050859-1284-4bf5-b05a-13846f83988f')
    def test_update_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.update_qos_policy, non_exist_id,
                          shared=False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('09e435b7-44d3-4f9d-8aa8-c295d46b5866')
    def test_delete_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.delete_qos_policy, non_exist_id)


class QosBandwidthLimitRuleNegativeTestJSON(base.BaseAdminNetworkTest):

    required_extensions = [qos_apidef.ALIAS]

    @decorators.attr(type='negative')
    @decorators.idempotent_id('e9ce8042-c828-4cb9-b1f1-85bd35e6553a')
    def test_rule_update_rule_nonexistent_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        policy = self.create_qos_policy(name='test-policy',
                                        description='test policy',
                                        shared=False)
        rule = self.create_qos_bandwidth_limit_rule(policy_id=policy['id'],
                                                    max_kbps=1,
                                                    max_burst_kbps=1)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.update_bandwidth_limit_rule,
            non_exist_id, rule['id'], max_kbps=200, max_burst_kbps=1337)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('a2c72066-0c32-4f28-be7f-78fa721588b6')
    def test_rule_update_rule_nonexistent_rule(self):
        non_exist_id = data_utils.rand_name('qos_rule')
        policy = self.create_qos_policy(name='test-policy',
                                        description='test policy',
                                        shared=False)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.update_bandwidth_limit_rule,
            policy['id'], non_exist_id, max_kbps=200, max_burst_kbps=1337)
